function [Centers,ColumNames,Shapes]=MyShapeDetection(Frame,GaussBlur,MethodThreshold,Threshold,AirMinDetection,Plot)
% Function to detect centroids on frames

if nargin<2
    GaussBlur=0; MethodThreshold=1; Threshold=0.99; Plot=0;
end

%% Gaussian blur application and 8 bit conversion 
if GaussBlur==0
    ImageGaussBlur=Frame;
else
    [~,d]=version; MatlabDate=str2double(d(end-4:end)); clear d;
    if MatlabDate>=2015
        ImageGaussBlur=imgaussfilt(Frame,GaussBlur);
    else
        GaussFilter=fspecial('gaussian',fix(size(Frame)/10),GaussBlur);
        ImageGaussBlur=imfilter(Frame,GaussFilter);
    end
end
[ny,nx]=size(Frame);

%% Threshold
if MethodThreshold==1
    [Counts,CenterBins]=hist(reshape(double(ImageGaussBlur),[ny*nx 1]),100);
    Counts=Counts'; CenterBins=CenterBins'; SumCumCounts=cumsum(Counts)/sum(Counts);
    I=find(SumCumCounts>Threshold); ThresholdValue=CenterBins(I(1));
elseif MethodThreshold==2
    ThresholdValue=Threshold*max(max(ImageGaussBlur));
elseif MethodThreshold==3
    ThresholdValue=Threshold;
end
ImageBW=ImageGaussBlur>ThresholdValue;
ImageThreshold=double(ImageGaussBlur).*ImageBW;

%% Center detection
Properties={'Centroid','WeightedCentroid',...
    'Area','Perimeter','MinorAxisLength','MajorAxisLength',...
    'Eccentricity','MeanIntensity','MaxIntensity','Orientation'...,
    'PixelList'};

s=regionprops(ImageBW,ImageThreshold,Properties); np=length(Properties);

ColumNames=cell(1,np-2); Centers=cell(1,np-2);
for i=2:np-1
    ColumNames{1,i-1}=Properties{1,i};
    Centers{1,i-1}=vertcat(s.(Properties{1,i}));
end
Centers=horzcat(Centers{1,:});

if isempty(Centers)==0
    I=find(Centers(:,3)>=AirMinDetection);
    if isempty(I)==0
        Centers=Centers(I,:);
        Shapes=cell(size(Centers,1),1);
        for i=1:size(Centers,1)
            Shapes{i}=s(I(i)).PixelList;
        end
    else
        Centers=[]; Shapes=[];
    end
else
    Shapes=[];
end

%% Visualization
if Plot==1
    scrsz=get(0,'ScreenSize');
    scrsz2=[(scrsz(3)-scrsz(4))/2 scrsz(2) scrsz(4) scrsz(4)];
    row=2; col=2;
    figure('name','Test 1','NumberTitle','off');
    set(gcf,'Position',scrsz2);
    subplot(row,col,1); imagesc(Frame);
    MyFigStyleLaTeX(0,0,0,'Original frame',2,'$I$',[1 1 1]);
    subplot(row,col,2); imagesc(ImageGaussBlur);
    MyFigStyleLaTeX(0,0,0,'Gaussian blur',2,'$I$',[1 1 1]);
    subplot(row,col,3); imagesc(ImageBW);
    MyFigStyleLaTeX(0,0,0,'Binary',2,'$I$',[1 1 1]);
    subplot(row,col,4); imagesc(ImageThreshold);
    t=linspace(0,2*pi,50);
    numObj = numel(s); hold on;
    for k = 1 : numObj
        
        xbarw = s(k).WeightedCentroid(1);
        ybarw = s(k).WeightedCentroid(2);
        xbar = s(k).Centroid(1);
        ybar = s(k).Centroid(2);
        
        plot(xbarw,ybar,'x','MarkerSize',10,'LineWidth',1,'color','r');
        plot(xbar,ybarw,'+','MarkerSize',10,'LineWidth',1,'color','y');

        a = s(k).MajorAxisLength/2;
        b = s(k).MinorAxisLength/2;
        theta = -s(k).Orientation*pi/180;
        xellipse = xbar + a*cos(t)*cos(theta) - b*sin(t)*sin(theta);
        yellipse = ybar + a*cos(t)*sin(theta) + b*sin(t)*cos(theta);
        
        plot(xellipse,yellipse,'r','LineWidth',1,'color','y');
        
    end; hold off;
    MyFigStyleLaTeX(0,0,0,'Threshold',2,'$I$',[1 1 1]);
    colormap gray;
elseif Plot==2
    scrsz=get(0,'ScreenSize'); row=2; col=2;
    figure('name','Test 1','NumberTitle','off');
    set(gcf,'Position',scrsz);
    imagesc(Frame); t=linspace(0,2*pi,50);
    numObj = numel(s); hold on;
    for k = 1 : numObj
        
        xbarw = s(k).WeightedCentroid(1);
        ybarw = s(k).WeightedCentroid(2);
        xbar = s(k).Centroid(1);
        ybar = s(k).Centroid(2);
        
        plot(xbarw,ybar,'x','MarkerSize',10,'LineWidth',2,'color','white');
        plot(xbar,ybarw,'+','MarkerSize',10,'LineWidth',2,'color','green');

        a = s(k).MajorAxisLength/2;
        b = s(k).MinorAxisLength/2;
        theta = -s(k).Orientation*pi/180;
        xellipse = xbar + a*cos(t)*cos(theta) - b*sin(t)*sin(theta);
        yellipse = ybar + a*cos(t)*sin(theta) + b*sin(t)*cos(theta);
        
        plot(xellipse,yellipse,'r','LineWidth',2,'color','black');
        
    end; hold off;
    MyFigStyleLaTeX(0,0,0,'Original frame',1,'$I$',[1 1 1]);
    colormap jet;
end



end