function [PositionsDisplacements,DeltaTime,MSD,AutoCorrelation]=Traj2AllDisp_TSfix(time,positions,ntau)
% This function calculates the displacements for a given trajectory
% Three quantities are associated to the displacements: the initial
% position, the difference between the final and the initial position and
% the mean value of the position during the displacement

% % Numeric trajectory as example to test the function
% np=100; dt=1; dl=10; ntau=50;
% t=(1:np)'; x=zeros(np,1); y=zeros(np,1);
% theta=(rand(1)-0.5)*2*pi;
% for i=2:np
%     if mod(i,dt)==0
%         theta=(rand(1)-0.5)*2*pi;
%     end
%     dx=cos(theta)*dl;
%     dy=sin(theta)*dl;
%     x(i)=x(i-1)+dx; y(i)=y(i-1)+dy;    
% end
% time=t; positions=[x y];
% clearvars -except time positions ntau

%% Function
% Creation of the matrix for displacements and position
[np,nd]=size(positions); % np = trajectory length and nd = dimension number
if nargin < 3
    ntau = np - 1;
end
if ntau > np - 1
    disp([num2str(ntau) ' time steps too large for a ' num2str(np - 1) ' time step trajectory!']);
    ntau = np - 1;
end
Mdiff=zeros(np,np,nd); % displacement matrix
Mi=zeros(np,np,nd); % initial position matrix
Mif=zeros(np,np,nd); % (initial+final)/2 position matrix
Mmean=zeros(np,np,nd); % mean position matrix (requier a loop)
Mprod=zeros(np,np,nd); % Product matrix for autocorrelation function
for i = 1 : nd
    Mdiff(:,:,i)=repmat(positions(:,i),[1 np 1])-repmat(positions(:,i)',[np 1 1]);
    Mi(:,:,i)=repmat(positions(:,i)',[np 1 1]);
    Mif(:,:,i)=(repmat(positions(:,i),[1 np 1])+repmat(positions(:,i)',[np 1 1]))/2;
    for r = 1 : np
        for c = 1 : np
            if r == c
                Mmean(r,c,i)=0;
            elseif r<c
                Mmean(r,c,i)=sum(positions(r:c,i))/length(positions(r:c,i));
            elseif r>c
                Mmean(r,c,i)=sum(positions(c:r,i))/length(positions(c:r,i));
            end
        end
    end    
    Mprod(:,:,i)=repmat(positions(:,i),[1 np 1]).*repmat(positions(:,i)',[np 1 1]);
end
Ti=repmat(time',[np 1 1]); % initial instant matrix
T2=repmat(time,[1 np 1])-repmat(time',[np 1 1]); % time step matrix

% Cell containing the displacement and position for all time steps
PositionsDisplacements=cell(1,ntau); % cell for displacements and positions
DeltaTime=zeros(ntau,1); % time steps
MSD=zeros(ntau,nd); % mean square displacement
AutoCorrelation=zeros(ntau,nd); % autocorrelation function
for i = 1 : ntau
    PositionsDisplacements{i}=zeros(np-i,nd*4+1);
    if length(unique(diag(T2(i+1:end,1:end-i))))==1
        DeltaTime(i)=unique(diag(T2(i+1:end,1:end-i)));
    elseif length(unique(diag(T2(i+1:end,1:end-i))))>1
        MeanTau=mean(diag(T2(i+1:end,1:end-i)));
        StdTau=std(diag(T2(i+1:end,1:end-i)));
        if 100*StdTau/MeanTau<1e-3
            DeltaTime(i)=mean(diag(T2(i+1:end,1:end-i)));
        else
            error(['tau = ' num2str(DeltaTime(i)) ' pm ' num2str(std(diag(T2(i+1:end,1:end-i))))]);
        end
    end
    for j=1:nd
        % local matrix
        Mdiffpart=Mdiff(i+1:end,1:end-i,j);
        Mipart=Mi(i+1:end,1:end-i,j);
        Mifpart=Mif(i+1:end,1:end-i,j);
        Mmeanpart=Mmean(i+1:end,1:end-i,j);
        Mprodpart=Mprod(i+1:end,1:end-i,j);
        % fill cells
        PositionsDisplacements{i}(:,j)=diag(Mipart); % initial positions
        PositionsDisplacements{i}(:,j+nd)=diag(Mifpart); % (initial+final)/2 positions
        PositionsDisplacements{i}(:,j+2*nd)=diag(Mmeanpart); % mean positions
        PositionsDisplacements{i}(:,j+3*nd)=diag(Mdiffpart); % displacements
        MSD(i,j)=mean(diag(Mdiffpart.^2)); % mean square displacement
        AutoCorrelation(i,j)=mean(diag(Mprodpart)); % autocorrelation
    end
    Tipart=Ti(i+1:end,1:end-i);
    PositionsDisplacements{i}(:,end)=diag(Tipart); % initial instants
end



end